#include <stdio.h>
#include "midas.h"
#include "msystem.h"

/* internal definitions */
#define  D16   16	 
#define  D24   24
#define  LOOP   1
#define  QUIT   2
#define  HELP   3
#define  SKIP   4
#define  JOB    5
#define  CHECK  6
#define  READ   7

/* structure for CAMAC definition */
typedef struct {
  INT       m;
  INT				b;
  INT				c;
  INT				n;
  INT				a;
  INT				f;
  WORD         d16;
  DWORD        d24;
  INT          r;
  INT          w;
  INT          q;
  INT          x;
  } CAMAC;

/* Default CAMAC definition */
CAMAC Prompt[2] ={ {24,0,1,2,3,0,0,0,1,0,0,0},
                   {0}
                 };

/* Default job file name */
char     job_name[128]="fcna.fcn";

HNDLE    hDB, hKey, hConn; 
FILE *   pF;
BOOL  jobflag;
char  addr[128];

/* prototype */
void fcnasub(INT mode);
void help_page(INT mode);
INT  decode_line (CAMAC *p, char * str);
INT  read_job_file(FILE * pF, INT action, void ** job, char * name);

/*--------------------------------------------------------------------*/
INT  read_job_file(FILE * pF, INT action, void ** job, char * name)
  {
    DWORD n, j;
    char  line[128];
    CAMAC * pjob;

    if (action == CHECK)
      {
        if (*name == 0)
          sprintf(name,"%s",job_name);
        pF = fopen(name, "r");
        if (pF)
          {
            fclose (pF);
            return JOB;
          }
        printf("FCNA-I- File not found :%s\n",name);
        return SKIP;
      }
    else
      {
        pF = fopen(name, "r");
        if (pF)
          {
            n = 0;
            while (fgets(line, 128, pF))
              n++;
            *job = malloc ((n+1)*sizeof(CAMAC));
            pjob = *job;
            memset ((char *)pjob, 0, (n+1)*sizeof(CAMAC));
            for (j=0;j<n;j++)
              memcpy((char *) pjob++, (char *)Prompt, sizeof(CAMAC));
            rewind (pF);
            pjob = *job;
            while (fgets(line, 128, pF))
              decode_line(pjob++, line);
            fclose (pF);
            return JOB;
          }
      }
      return SKIP;
  }

/*--------------------------------------------------------------------*/
void fcnasub(INT mode)
{
	char str[128], line[128];
	INT  status, j;
  DWORD  size, cmd;
  CAMAC *P, *p, *job;

	/* Loop return label */
	if (jobflag)
	{
	jobflag = FALSE;
 }

/* Load default CAMAC */
  P = Prompt;
  while(1)
	{
        if (P->m == D24)
          sprintf(addr,"B%1.1dC%1.1dN%2.2dA%2.2dF%2.2d [%d/0x%6.6x Q%1.1dX%1.1d] R%dW%dM%2d"
                 ,P->b,P->c,P->n,P->a,P->f,P->d24,P->d24,P->q,P->x,P->r,P->w,P->m);
	      else
          sprintf(addr,"B%1.1dC%1.1dN%2.2dA%2.2dF%2.2d [%d/0x%4.4x Q%1.1dX%1.1d] R%dW%dM%2d"
                 ,P->b,P->c,P->n,P->a,P->f,P->d16,P->d16,P->q,P->x,P->r,P->w,P->m);

    /* prompt */
    printf("FCNA> [%s] :",addr);
		gets(str);

    /* decode line */    
		status = decode_line(P, str);
		if (status == QUIT)
			return;
    else if (status == HELP)
      help_page(mode);
    else if (status == JOB)
      {
        printf("\nFCNA> Job file name [%s]:",job_name);
        gets (line);
        status = read_job_file(pF, CHECK, &job, line);
        if (status == JOB)
          {
            printf("%s",job_name);
            status=read_job_file(pF, READ, &job, job_name);
          }
      }
    if (status == LOOP || status == JOB)
      {
        if (status == LOOP)
          p = P;
        if (status == JOB)
          p = job;
        while (p->m)
          {
            for (j=0;j<p->r;j++)
              {
                    /* compute cmd request */
                    if (p->f < 16)
                      if (p->m == D24)
                        cmd = FCNA_I24;
                      else
                        cmd = FCNA_I16;
                    else if (p->f <32)
                      if (p->m == D24)
                        cmd = FCNA_O24;
                      else
                        cmd = FCNA_O16;
                    size = sizeof(DWORD);
                    if (p->m == D24)
                      { /* Actual 24 bits CAMAC operation */
                          status = rpc_call(hConn, RPC_FCNA
                          , cmd, p->b, p->c, p->n, p->a, p->f
                          , &p->d24, &size, &p->q, &p->x);
                          if (status == RPC_SUCCESS)
                            sprintf(addr,"B%1.1dC%1.1dN%2.2dA%2.2dF%2.2d [%d/0x%6.6x Q%1.1dX%1.1d] R%dW%dM%2d"
		                          ,p->b,p->c,p->n,p->a,p->f,p->d24,p->d24,p->q,p->x,p->r,p->w,p->m);
                          else
                            printf("\nfcna-E- RPC error :%d\n",status);
                      }
	                  else
                      { /* Actual 16 bits CAMAC operation */
                          status = rpc_call(hConn, RPC_FCNA
                          , cmd, p->b, p->c, p->n, p->a, p->f
                          , (DWORD *)(&p->d16), &size, &p->q, &p->x);
                          if (status == RPC_SUCCESS)
                              sprintf(addr,"B%1.1dC%1.1dN%2.2dA%2.2dF%2.2d [%d/0x%4.4x Q%1.1dX%1.1d] R%dW%dM%2d"
		                          ,p->b,p->c,p->n,p->a,p->f,p->d16,p->d16,p->q,p->x,p->r,p->w,p->m);
                          else
                            printf("\nfcna-E- RPC error :%d\n",status);
                      }
                /* Result display */
                if (p->r > 1)
                  printf("\nFCNA> [%s]",addr);
                if (j==p->r)
                  break;
                if (p->w != 0)
                  ss_sleep(p->w);
              }
              if (p->r > 1)
                printf("\n");
            p++; 
          };
        if (status == JOB)
          free (job);
      }
   }
}

/*--------------------------------------------------------------------*/
INT decode_line (CAMAC *P, char * ss)
{
	char * c, * cmd;
	char pp[128], *p, *s, *ps;
  DWORD  tmp;
  BOOL   ok=FALSE;

	p = pp;
	s = ss;
	memset (pp,0,128);

  /* convert to uppercase */
  c = ss;
  while (*c)
    *c++ = toupper(*c);
	
	while (*s)
  {
		if ((*s == 'B') || (*s == 'C') ||
			  (*s == 'N') || (*s == 'A') ||
			  (*s == 'F') || (*s == 'C') ||
			  (*s == 'R') || (*s == 'W') || 
			  (*s == 'G') || (*s == 'H') ||
			  (*s == 'E') || (*s == 'Q'))
          *p++ = ' ';
		else if ((*s == 'X') || (*s == 'D') || (*s == 'O'))
      {
        *p++ = ' ';
     	  *p++ = *s++;
        while (*s)
          *p++ = *s++;
        break;
      };
      *p++ = *s++;
  }
	*p++ = ' ';

	p = pp;

	if (cmd = strpbrk(p,"G"))
		return LOOP;
	if (cmd = strpbrk(p,"H"))
    return HELP;
	if (cmd = strpbrk(p,"QE"))
		return QUIT;
	if (cmd = strpbrk(p,"J"))
    return JOB;
	if (cmd = strpbrk(p,"D"))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
		if (P->m == D24)
      {
        tmp = strtoul((cmd+1),NULL,10);
        if (tmp>=0x0 && tmp<=0xffffff)
          {
    			  P->d24 = tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range 0:16777215\n");
      }
		else
      {
        tmp = strtoul((cmd+1),NULL,8);
        if (tmp>=0x0 && tmp<=0xffff)
          {
            P->d16 = (WORD) tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range 0:65535\n");
      }
		*ps = ' '; 
	}
	if (cmd = strpbrk(p,"X"))
	{
		ps = strchr(cmd,' ');
		*ps = 0;       
		if (P->m == D24)
      {
        tmp = strtoul((cmd+1),NULL,16);
        if (tmp>=0x0 && tmp<=0xffffff)
          {
            P->d24 = tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range 0x0:0xffffff\n");
      }
		else
      {
        tmp = strtoul((cmd+1),NULL,16);
        if (tmp>=0x0 && tmp<=0xffff)
          {
            P->d16 = (WORD) tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range 0x0:0xffff\n");
      }
		*ps = ' '; 
	}
	if (cmd = strpbrk(p,"O"))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
		if (P->m == D24)
      {
        tmp = strtoul((cmd+1),NULL,8);
        if (tmp>=0 && tmp<=077777777)
          {
            P->d24 = tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range O0:O77777777\n");
      }
		else
      {
        tmp = strtoul((cmd+1),NULL,8);
        if (tmp>=00 && tmp<=0177777)
          {
            P->d16 = (WORD) tmp;
            ok = TRUE;
          }
        else
          printf("mfcna-E- Data out of range O0:O177777\n");
      }
    *ps = ' '; 
	}
  if (cmd = strchr(p,'B'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<8 && tmp>=0)
      {
        P->b = tmp;
        ok = TRUE;
      }
    else
      printf("mfcna-E- B out of range 0:7\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'C'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<8 && tmp>=0)
      {
        P->c = tmp;
        ok = TRUE;
      }
    else
      printf("mfcna-E- C out of range 0:7\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'N'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<32 && tmp>=0)
      {
        P->n = tmp;
        ok = TRUE;
      }
    else
      printf("mfcna-E- N out of range 0:31\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'A'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<16 && tmp>=0)
      {
        P->a = tmp;
        ok = TRUE;
      }
    else
      printf("mfcna-E- A out of range 0:15\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'F'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<32 && tmp>=0)
      {
        P->f = tmp;
        ok = TRUE;
      }
    else
      printf("mfcna-E- F out of range 0:31\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'R'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<1000 && tmp>=0)
      {
        if (tmp==0)
          tmp = 1;
        P->r = tmp;
      }
    else
      printf("mfcna-E- R out of range 1:1000\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'W'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp<10001 || tmp>=0)
      P->w = tmp;
    else
      printf("mfcna-E- W out of range 0:10000\n");
		*ps = ' '; 
	}
	if (cmd = strchr(p,'M'))
	{
		ps = strchr(cmd,' ');
		*ps = 0;
    tmp = atoi((cmd+1));
    if (tmp==D16 || tmp==D24)
      P->m = tmp;
    else
      printf("mfcna-E- M out of range 16,24\n");
		*ps = ' '; 
	}
  if (ok)
    return LOOP;
	return SKIP;
}

/*--------------------------------------------------------------------*/
void help_page(INT mode)
  {
        printf("\n*-v%1.2lf----------- H E L P   F C N A-------------------*\n"
          ,cm_get_version()/100.0);
        printf("          Interactive Midas CAMAC command\n");
        printf("          ===============================\n");
        printf("\n");
        printf(" Outputs: Data [dec/hex X=0/1 - Q=0/1 ]\n");
        printf(" Inputs : Bx :  Branch   [0 -  7]    Cx :  Crate    [0 -  7]\n");
        printf("          Nx :  Slot     [1 - 31]    Ax :  SubAdd.  [0 - 15]\n");
        printf("          Fx :  Function [0 - 31]\n");
        printf("           H :  HELP on   FCNA      Q/E :  Quit/Exit from FCNA\n");
        printf("          Rx :  Repetition counter               [1 -  1000]\n");
        printf("          Wx :  Delay between command (ms)       [0 - 10000]\n");
        printf("           J :  Perform JOB (command list from file )\n");
        printf("                Same syntax as in inteactive session\n");
        printf("           G :  PERFORM ACTION of predefine set-up\n");
        printf("           D :  Decimal     Data     [0 - max data_size]\n");
        printf("           O :  Octal       Data     [0 - max data_size]\n");
        printf("           X :  Hexadecimal Data     [0 - max data_size]\n");
        printf("\n");
        printf(">>>>>>>>>> Data has to be given LAST if needed in the command string <<<<<\n");
        printf("\n");
        printf(" examples: ");
        printf(">mfcna -cl ""triggerFE"" -ho local\n");
        printf("      FCNA> [B0C1N30A00F00 [0/0x000000 Q0X0] R1W0] :n30a9f24\n");
        printf("      FCNA> [B0C1N06A00F24 [0/0x000000 Q1X1] R1W0] :n6f9a0\n");
        printf("      FCNA> [B0C1N06A00F09 [0/0x000000 Q1X1] R1W0] :f25\n");
        printf("      FCNA> [B0C1N06A00F25 [0/0x000000 Q1X1] R1W0] :f0\n");
        printf("      FCNA> [B0C1N06A00F00 [345/0x000159 Q1X1] R1W0] :a1\n");
        printf("      FCNA> [B0C1N06A01F00 [365/0x00016D Q1X1] R1W0] :a2\n");
        printf("      FCNA> [B0C1N06A02F00 [374/0x000176 Q1X1] R1W0] :\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R1W0] :r3\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R3W0] :g\n");
        printf("\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R3W0]\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R3W0]\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R3W0]\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R3W0] :r\n");
        printf("      FCNA> [B0C1N06A02F00 [0/0x000176 Q1X1] R1W0] :n3f16x77\n");
        printf("      FCNA> [B0C1N03A00F16 [0/0x000077 Q1X1] R1W0] :a0f16d63\n");
        printf("      FCNA> [B0C1N02A00F16 [63/0x00003F Q1X1] R1W0] :n2f16o63\n");
        printf("      FCNA> [B0C1N02A00F16 [51/0x000033 Q1X1] R1W0] :\n");
 }

/*--------------------------------------------------------------------*/
int main(unsigned int argc, char **argv)
{
  char   host_name[30], expt_name[30], client_name[256];
  INT    status, mode=0;
  DWORD   i;
  BOOL   debug;
  
  /* set default */
  host_name[0] = '\0';
  expt_name[0] = '\0';
  client_name[0] = '\0';
  /* get parameters */
  /* parse command line parameters */
  for (i=1 ; i<argc ; i++)
    {
    if (argv[i][0] == '-' && argv[i][1] == 'd')
      debug = TRUE;
    else if (argv[i][0] == '-')
      {
      if (i+1 >= argc || argv[i+1][0] == '-')
        goto usage;
      if (strncmp(argv[i],"-e",3) == 0)
      	strcpy(expt_name, argv[++i]);
      else if (strncmp(argv[i],"-h",3)==0)
        strcpy(host_name, argv[++i]);
      else if (strncmp(argv[i],"-c",3)==0)
        strcpy(client_name, argv[++i]);
      else
        {
usage:
        printf("usage: mfcna -c Client [-h Hostname] [-e Experiment]\n\n");
        return 0;
        }
      }
    }

  if (client_name[0] == '\0')
    {
      printf("Client unknown, please specify the Midas CAMAC frontend (-c ...)\n");
      goto usage;
    }
  

  /* turn off message display, turn on message logging */
  cm_set_msg_print(MT_ALL, 0, NULL);

  /* connect to experiment */
  status = cm_connect_experiment(host_name, expt_name, "mfcna", 0);
  status = CM_SUCCESS;
  if (status != CM_SUCCESS)
    {
      printf("Cannot connect to host '%s'.\n", host_name);
      return 0;
    }
  
 if (debug)
   cm_set_watchdog_params(TRUE, 0);
  
 /* connect to the database */
 cm_get_experiment_database(&hDB, &hKey);
  
  /* register FCNA_RPC call */
 if (cm_connect_client(client_name, &hConn) == CM_SUCCESS)
    {
    DWORD data, size, q, x;

    /* test if FCNA function implemented */
    size = sizeof(WORD);
    status = rpc_call(hConn, RPC_FCNA, FCNA_I16, 0, 0, 0, 0, 0, &data, &size, &q, &x);

    if (status != RPC_SUCCESS)
      printf("FCNA functionality not implemented by client %s\n", client_name);
    else
      fcnasub(mode);

    printf("mfcna- Thank you!\n");
    cm_disconnect_client(hConn, FALSE);
    }
 else
   printf("Cannot connect to client %s\n",client_name);
 cm_disconnect_experiment();
}